@@ -35,6 +35,7 @@
 #include <gtk/gtkmenuitem.h>
 #include <gtk/gtktreeview.h>
 #include <gtk/gtkwindow.h>
+#include <gtk/gtkviewport.h>
 #include <eel/eel-gdk-extensions.h>
 
 #define EEL_DEFAULT_POPUP_MENU_DISPLACEMENT 	2
@@ -149,4 +150,11 @@ gboolean              eel_gtk_tree_view_cell_is_completely_visible    (GtkTreeVi
 								       GtkTreePath          *path,
 								       GtkTreeViewColumn    *column);
 
+/* GtkViewport */
+gboolean              eel_gtk_viewport_get_visible_rect               (GtkViewport          *viewport, 
+								       GdkRectangle         *rect);
+
+void                  eel_gtk_viewport_scroll_to_rect                 (GtkViewport          *viewport,
+								       GdkRectangle         *rect);
+
 #endif /* EEL_GTK_EXTENSIONS_H */
