@@ -31,11 +31,16 @@
 #include "eel-gtk-container.h"
 #include "eel-gtk-extensions.h"
 #include "eel-gtk-macros.h"
+#include "eel-accessibility.h"
 #include <gtk/gtkbutton.h>
 #include <gtk/gtkcheckbutton.h>
 #include <gtk/gtkimage.h>
 #include <gtk/gtksignal.h>
 #include <gtk/gtktogglebutton.h>
+#include <gtk/gtkbindings.h>
+#include <gtk/gtkaccessible.h>
+#include <gdk/gdkkeysyms.h>
+#include <atk/atkimage.h>
 
 #define DEFAULT_SPACING 0
 #define DEFAULT_X_PADDING 0
@@ -43,6 +48,13 @@
 #define DEFAULT_X_ALIGNMENT 0.5
 #define DEFAULT_Y_ALIGNMENT 0.5
 
+/* Signals */
+enum
+{
+	ACTIVATE,
+	LAST_SIGNAL
+};
+
 /* Arguments */
 enum
 {
@@ -77,8 +89,14 @@ struct EelLabeledImageDetails
 	gboolean fill;
 };
 
-static void          eel_labeled_image_class_init   (EelLabeledImageClass  *labeled_image_class);
-static void          eel_labeled_image_init         (EelLabeledImage       *image);
+/* derived types so we can add our accessibility interfaces */
+static GType         eel_labeled_image_button_get_type        (void);
+static GType         eel_labeled_image_check_button_get_type  (void);
+static GType         eel_labeled_image_toggle_button_get_type (void);
+
+
+static void          eel_labeled_image_class_init         (EelLabeledImageClass  *labeled_image_class);
+static void          eel_labeled_image_init               (EelLabeledImage       *image);
 static void          eel_labeled_image_finalize           (GObject               *object);
 
 /* GObjectClass methods */
@@ -99,6 +117,7 @@ static void          eel_labeled_image_size_allocate      (GtkWidget
 							   GtkAllocation         *allocation);
 static void          eel_labeled_image_map                (GtkWidget             *widget);
 static void          eel_labeled_image_unmap              (GtkWidget             *widget);
+static AtkObject    *eel_labeled_image_get_accessible     (GtkWidget             *widget);
 
 /* GtkContainerClass methods */
 static void          eel_labeled_image_add                (GtkContainer          *container,
@@ -121,6 +140,8 @@ static void          labeled_image_update_alignments      (EelLabeledImage
 static gboolean      labeled_image_show_label             (const EelLabeledImage *labeled_image);
 static gboolean      labeled_image_show_image             (const EelLabeledImage *labeled_image);
 
+static guint labeled_image_signals[LAST_SIGNAL] = { 0 };
+
 EEL_CLASS_BOILERPLATE (EelLabeledImage, eel_labeled_image, GTK_TYPE_CONTAINER)
 
 /* Class init methods */
@@ -130,6 +151,7 @@ eel_labeled_image_class_init (EelLabeledImageClass *labeled_image_class)
 	GObjectClass *gobject_class = G_OBJECT_CLASS (labeled_image_class);
 	GtkWidgetClass *widget_class = GTK_WIDGET_CLASS (labeled_image_class);
 	GtkContainerClass *container_class = GTK_CONTAINER_CLASS (labeled_image_class);
+	GtkBindingSet *binding_set;
 
 	gobject_class->finalize = eel_labeled_image_finalize;
 
@@ -143,12 +165,37 @@ eel_labeled_image_class_init (EelLabeledImageClass *labeled_image_class)
  	widget_class->expose_event = eel_labeled_image_expose_event;
 	widget_class->map = eel_labeled_image_map;
 	widget_class->unmap = eel_labeled_image_unmap;
+	widget_class->get_accessible = eel_labeled_image_get_accessible;
 
  	/* GtkContainerClass */
 	container_class->add = eel_labeled_image_add;
 	container_class->remove = eel_labeled_image_remove;
 	container_class->forall = eel_labeled_image_forall;
-  
+
+	labeled_image_signals[ACTIVATE] =
+		g_signal_new ("activate",
+			      G_TYPE_FROM_CLASS (labeled_image_class),
+			      G_SIGNAL_RUN_LAST | G_SIGNAL_ACTION,
+			      G_STRUCT_OFFSET (EelLabeledImageClass,
+					       activate),
+			      NULL, NULL, 
+			      g_cclosure_marshal_VOID__VOID,
+			      G_TYPE_NONE, 0);	
+	widget_class->activate_signal = labeled_image_signals[ACTIVATE];
+
+	binding_set = gtk_binding_set_by_class (gobject_class);
+	
+	gtk_binding_entry_add_signal (binding_set, 
+				      GDK_Return, 0,
+				      "activate", 0);
+	gtk_binding_entry_add_signal (binding_set, 
+				      GDK_KP_Enter, 0,
+				      "activate", 0);
+	gtk_binding_entry_add_signal (binding_set, 
+				      GDK_space, 0,
+				      "activate", 0);
+
+
 	/* Properties */
         g_object_class_install_property (
 		gobject_class,
@@ -462,13 +509,30 @@ eel_labeled_image_expose_event (GtkWidget *widget,
 				GdkEventExpose *event)
 {
 	EelLabeledImage *labeled_image;
+	ArtIRect label_bounds;
 
 	g_return_val_if_fail (EEL_IS_LABELED_IMAGE (widget), TRUE);
 	g_return_val_if_fail (GTK_WIDGET_REALIZED (widget), TRUE);
 	g_return_val_if_fail (event != NULL, TRUE);
 
   	labeled_image = EEL_LABELED_IMAGE (widget);
-	
+
+	if (GTK_WIDGET_STATE (widget) == GTK_STATE_SELECTED || 
+	    GTK_WIDGET_STATE (widget) == GTK_STATE_ACTIVE) {
+		label_bounds = eel_labeled_image_get_label_bounds (EEL_LABELED_IMAGE (widget));
+
+		gtk_paint_flat_box (widget->style,
+				    widget->window,
+				    GTK_WIDGET_STATE (widget),
+				    GTK_SHADOW_NONE,
+				    &event->area,
+				    widget,
+				    "eel-labeled-image",
+				    label_bounds.x0, label_bounds.y0,
+				    label_bounds.x1 - label_bounds.x0, 
+				    label_bounds.y1 - label_bounds.y0);
+	}
+
 	if (labeled_image_show_label (labeled_image)) {
 		eel_gtk_container_child_expose_event (GTK_CONTAINER (widget),
 						      labeled_image->details->label,
@@ -481,6 +545,17 @@ eel_labeled_image_expose_event (GtkWidget *widget,
 						      event);
 	}
 
+	if (GTK_WIDGET_HAS_FOCUS (widget)) {
+		label_bounds = eel_labeled_image_get_image_bounds (EEL_LABELED_IMAGE (widget));
+		gtk_paint_focus (widget->style, widget->window,
+				 GTK_STATE_NORMAL,
+				 &event->area, widget,
+				 "eel-focusable-labeled-image",
+				 label_bounds.x0, label_bounds.y0,
+				 label_bounds.x1 - label_bounds.x0, 
+				 label_bounds.y1 - label_bounds.y0);
+	}		
+
 	return FALSE;
 }
 
@@ -1340,6 +1415,44 @@ eel_labeled_image_set_fixed_image_height (EelLabeledImage *labeled_image,
 	gtk_widget_queue_resize (GTK_WIDGET (labeled_image));
 }
 
+/**
+ * eel_labeled_image_set_selected:
+ * @labeled_image: A EelLabeledImage.
+ * @selected: A boolean value indicating whether the labeled image
+ * should be selected.
+ *
+ * Selects or deselects the labeled image.
+ *
+ */
+void            
+eel_labeled_image_set_selected (EelLabeledImage *labeled_image,
+				gboolean selected)
+{
+	GtkStateType state;
+	g_return_if_fail (EEL_IS_LABELED_IMAGE (labeled_image));
+
+	state = selected ? GTK_STATE_SELECTED : GTK_STATE_NORMAL;
+
+	gtk_widget_set_state (GTK_WIDGET (labeled_image), state);
+	gtk_widget_set_state (labeled_image->details->image, state);
+	gtk_widget_set_state (labeled_image->details->label, state);
+}
+
+/**
+ * eel_labeled_image_get_selected:
+ * @labeled_image: A EelLabeledImage.
+ *
+ * Returns the selected state of the labeled image.
+ *
+ */
+gboolean        
+eel_labeled_image_get_selected (EelLabeledImage *labeled_image)
+{
+	g_return_val_if_fail (EEL_IS_LABELED_IMAGE (labeled_image), FALSE);
+
+	return GTK_WIDGET (labeled_image)->state == GTK_STATE_SELECTED;
+}
+
 /**
  * eel_labeled_image_set_spacing:
  * @labeled_image: A EelLabeledImage.
@@ -1597,7 +1710,7 @@ eel_labeled_image_button_new (const char *text,
 	GtkWidget *button;
 	GtkWidget *labeled_image;
 	
-	button = gtk_button_new ();
+	button = g_object_new (eel_labeled_image_button_get_type (), NULL);
 	labeled_image = eel_labeled_image_new (text, pixbuf);
 	gtk_container_add (GTK_CONTAINER (button), labeled_image);
 	gtk_widget_show (labeled_image);
@@ -1622,7 +1735,7 @@ eel_labeled_image_button_new_from_file_name (const char *text,
 
 	g_return_val_if_fail (pixbuf_file_name != NULL, NULL);
 	
-	button = gtk_button_new ();
+	button = g_object_new (eel_labeled_image_button_get_type (), NULL);
 	labeled_image = eel_labeled_image_new_from_file_name (text, pixbuf_file_name);
 	gtk_container_add (GTK_CONTAINER (button), labeled_image);
 	gtk_widget_show (labeled_image);
@@ -1645,7 +1758,7 @@ eel_labeled_image_toggle_button_new (const char *text,
 	GtkWidget *toggle_button;
 	GtkWidget *labeled_image;
 	
-	toggle_button = gtk_toggle_button_new ();
+	toggle_button = g_object_new (eel_labeled_image_toggle_button_get_type (), NULL);
 	labeled_image = eel_labeled_image_new (text, pixbuf);
 	gtk_container_add (GTK_CONTAINER (toggle_button), labeled_image);
 	gtk_widget_show (labeled_image);
@@ -1670,7 +1783,7 @@ eel_labeled_image_toggle_button_new_from_file_name (const char *text,
 
 	g_return_val_if_fail (pixbuf_file_name != NULL, NULL);
 	
-	toggle_button = gtk_toggle_button_new ();
+	toggle_button = g_object_new (eel_labeled_image_toggle_button_get_type (), NULL);
 	labeled_image = eel_labeled_image_new_from_file_name (text, pixbuf_file_name);
 	gtk_container_add (GTK_CONTAINER (toggle_button), labeled_image);
 	gtk_widget_show (labeled_image);
@@ -1738,7 +1851,7 @@ eel_labeled_image_check_button_new (const char *text,
 	GtkWidget *check_button;
 	GtkWidget *labeled_image;
 	
-	check_button = gtk_check_button_new ();
+	check_button = g_object_new (eel_labeled_image_check_button_get_type (), NULL);
 	labeled_image = eel_labeled_image_new (text, pixbuf);
 	gtk_container_add (GTK_CONTAINER (check_button), labeled_image);
 	gtk_widget_show (labeled_image);
@@ -1772,7 +1885,7 @@ eel_labeled_image_check_button_new_from_file_name (const char *text,
 
 	g_return_val_if_fail (pixbuf_file_name != NULL, NULL);
 	
-	check_button = gtk_check_button_new ();
+	check_button = g_object_new (eel_labeled_image_check_button_get_type (), NULL);
 	labeled_image = eel_labeled_image_new_from_file_name (text, pixbuf_file_name);
 	gtk_container_add (GTK_CONTAINER (check_button), labeled_image);
 	gtk_widget_show (labeled_image);
@@ -1870,3 +1983,235 @@ eel_labeled_image_get_text (const EelLabeledImage *labeled_image)
 
 	return g_strdup (gtk_label_get_text (GTK_LABEL (labeled_image->details->label)));
 }
+
+void
+eel_labeled_image_set_can_focus (EelLabeledImage *labeled_image,
+				 gboolean         can_focus)
+{
+	if (can_focus) {
+		GTK_WIDGET_SET_FLAGS
+			(GTK_WIDGET (labeled_image), GTK_CAN_FOCUS);
+		
+	} else {
+		GTK_WIDGET_UNSET_FLAGS
+			(GTK_WIDGET (labeled_image), GTK_CAN_FOCUS);
+	}
+}
+
+static AtkObjectClass *a11y_parent_class = NULL;
+
+static void
+eel_labeled_image_accessible_initialize (AtkObject *accessible,
+					 gpointer   widget)
+{
+	a11y_parent_class->initialize (accessible, widget);
+}
+
+static EelLabeledImage *
+get_image (gpointer object)
+{
+	GtkWidget *widget;
+
+	if (!(widget = GTK_ACCESSIBLE (object)->widget)) {
+		return NULL;
+	}
+	
+	if (GTK_IS_BUTTON (widget))
+		widget = GTK_BIN (widget)->child;
+
+	return EEL_LABELED_IMAGE (widget);
+}
+
+static G_CONST_RETURN gchar *
+eel_labeled_image_accessible_get_name (AtkObject *accessible)
+{
+	EelLabeledImage *labeled_image;
+
+	labeled_image = get_image (accessible);
+
+	if (labeled_image->details->label)
+		return gtk_label_get_text
+			(GTK_LABEL (labeled_image->details->label));
+
+	g_warning ("no label on '%p'", labeled_image);
+
+	return NULL;
+}
+
+static void
+eel_labeled_image_accessible_image_get_size (AtkImage *image,
+					     gint     *width,
+					     gint     *height)
+{
+	EelLabeledImage *labeled_image;
+
+	labeled_image = get_image (image);
+
+	if (!labeled_image || !labeled_image->details->image) {
+		*width = *height = 0;
+		return;
+	}
+
+	*width = labeled_image->details->image->allocation.width;
+	*height = labeled_image->details->image->allocation.height;
+}
+
+static void
+eel_labeled_image_accessible_image_interface_init (AtkImageIface *iface)
+{
+	iface->get_image_size = eel_labeled_image_accessible_image_get_size;
+}
+
+static void
+eel_labeled_image_accessible_class_init (AtkObjectClass *klass)
+{
+	a11y_parent_class = g_type_class_peek_parent (klass);
+
+	klass->get_name = eel_labeled_image_accessible_get_name;
+	klass->initialize = eel_labeled_image_accessible_initialize;
+}
+
+enum {
+	BUTTON,
+	CHECK,
+	TOGGLE,
+	PLAIN
+};
+
+static AtkObject *
+eel_labeled_image_get_accessible (GtkWidget *widget)
+{
+	int i;
+	static GType types[4] = { 0 };
+	const char *tname;
+	AtkRole role;
+	AtkObject *accessible;
+
+	if ((accessible = eel_accessibility_get_atk_object (widget)))
+		return accessible;
+
+	if (GTK_IS_CHECK_BUTTON (widget)) {
+		i = BUTTON;
+		role = ATK_ROLE_CHECK_BOX;
+		tname = "EelLabeledImageCheckButtonAccessible";
+
+	} else if (GTK_IS_TOGGLE_BUTTON (widget)) {
+		i = CHECK;
+		role = ATK_ROLE_TOGGLE_BUTTON;
+		tname = "EelLabeledImageToggleButtonAccessible";
+
+	} else if (GTK_IS_BUTTON (widget)) {
+		i = TOGGLE;
+		role = ATK_ROLE_PUSH_BUTTON;
+		tname = "EelLabeledImagePushButtonAccessible";
+
+	} else { /* plain */
+		i = PLAIN;
+		role = ATK_ROLE_IMAGE;
+		tname = "EelLabeledImagePlainAccessible";
+	}
+
+	if (!types [i]) {
+		static const GInterfaceInfo atk_image_info = {
+			(GInterfaceInitFunc) eel_labeled_image_accessible_image_interface_init,
+			(GInterfaceFinalizeFunc) NULL,
+			NULL
+		};
+
+		types [i] = eel_accessibility_create_derived_type
+			(tname, G_TYPE_FROM_INSTANCE (widget),
+			 eel_labeled_image_accessible_class_init);
+
+		g_type_add_interface_static (
+			types [i], ATK_TYPE_IMAGE, &atk_image_info);
+	}
+
+	accessible = g_object_new (types [i], NULL);
+	atk_object_set_role (accessible, role);
+
+	return eel_accessibility_set_atk_object_return (widget, accessible);
+}
+
+static void
+eel_labeled_image_button_class_init (GtkWidgetClass *klass)
+{
+	klass->get_accessible = eel_labeled_image_get_accessible;
+}
+
+static GType
+eel_labeled_image_button_get_type (void)
+{
+	static GType type = 0;
+
+	if (!type) {
+		GTypeInfo info = {
+			sizeof (GtkButtonClass),
+			(GBaseInitFunc) NULL,
+			(GBaseFinalizeFunc) NULL,
+			(GClassInitFunc) eel_labeled_image_button_class_init,
+			NULL, /* class_finalize */
+			NULL, /* class_data */
+			sizeof (GtkButton),
+			0, /* n_preallocs */
+			(GInstanceInitFunc) NULL
+		};
+		
+		type = g_type_register_static
+			(GTK_TYPE_BUTTON,
+			 "EelLabeledImageButton", &info, 0);
+	}
+
+	return type;
+}
+
+static GType
+eel_labeled_image_check_button_get_type (void)
+{
+	static GType type = 0;
+
+	if (!type) {
+		GTypeInfo info = {
+			sizeof (GtkCheckButtonClass),
+			(GBaseInitFunc) NULL,
+			(GBaseFinalizeFunc) NULL,
+			(GClassInitFunc) eel_labeled_image_button_class_init,
+			NULL, /* class_finalize */
+			NULL, /* class_data */
+			sizeof (GtkCheckButton),
+			0, /* n_preallocs */
+			(GInstanceInitFunc) NULL
+		};
+		
+		type = g_type_register_static
+			(GTK_TYPE_CHECK_BUTTON,
+			 "EelLabeledImageCheckButton", &info, 0);
+	}
+
+	return type;
+}
+
+static GType
+eel_labeled_image_toggle_button_get_type (void)
+{
+	static GType type = 0;
+
+	if (!type) {
+		GTypeInfo info = {
+			sizeof (GtkToggleButtonClass),
+			(GBaseInitFunc) NULL,
+			(GBaseFinalizeFunc) NULL,
+			(GClassInitFunc) eel_labeled_image_button_class_init,
+			NULL, /* class_finalize */
+			NULL, /* class_data */
+			sizeof (GtkToggleButton),
+			0, /* n_preallocs */
+			(GInstanceInitFunc) NULL
+		};
+		
+		type = g_type_register_static
+			(GTK_TYPE_TOGGLE_BUTTON,
+			 "EelLabeledImageToggleButton", &info, 0);
+	}
+
+	return type;
+}
