@@ -600,14 +600,19 @@ preferences_item_update_enumeration_menu (EelPreferencesItem *item)
 	char *enumeration_id;
 	
 	g_return_if_fail (EEL_IS_PREFERENCES_ITEM (item));
-	g_return_if_fail (item->details->item_type == EEL_PREFERENCE_ITEM_ENUMERATION_MENU);
+	g_return_if_fail (item->details->item_type == EEL_PREFERENCE_ITEM_ENUMERATION_MENU ||
+			  item->details->item_type == EEL_PREFERENCE_ITEM_ENUMERATION_MENU_INTEGER);
 
 	enumeration_id = eel_preferences_get_enumeration_id (item->details->preference_name);
-
+		
 	g_return_if_fail (eel_strlen (enumeration_id) > 0);
 	g_return_if_fail (eel_enumeration_id_get_length (enumeration_id) > 0);
 
-	current_value = eel_preferences_get_enum (item->details->preference_name);
+	if (item->details->item_type == EEL_PREFERENCE_ITEM_ENUMERATION_MENU) {
+		current_value = eel_preferences_get_enum (item->details->preference_name);
+	} else {
+		current_value = eel_preferences_get_integer (item->details->preference_name);
+	}
 
 	position = eel_enumeration_id_get_value_position (enumeration_id,
 							  current_value);
@@ -797,6 +802,7 @@ eel_preferences_item_new (const char *preference_name,
 		break;	
 
 	case EEL_PREFERENCE_ITEM_ENUMERATION_MENU:
+	case EEL_PREFERENCE_ITEM_ENUMERATION_MENU_INTEGER:
 		preferences_item_create_enumeration_menu (item);
 		break;	
 
@@ -943,8 +949,12 @@ enumeration_menu_changed_callback (EelStringPicker *string_picker,
 	
 	new_value = eel_enumeration_id_get_nth_value (enumeration_id,
 						      position);
-	
-	eel_preferences_set_enum (item->details->preference_name, new_value);
+
+	if (item->details->item_type == EEL_PREFERENCE_ITEM_ENUMERATION_MENU) {
+		eel_preferences_set_enum (item->details->preference_name, new_value);
+	} else {
+		eel_preferences_set_integer (item->details->preference_name, new_value);
+	}
 
  	g_free (enumeration_id);
 }
@@ -1065,6 +1075,7 @@ preferences_item_update_displayed_value (EelPreferencesItem *item)
 		break;
 
 	case EEL_PREFERENCE_ITEM_ENUMERATION_MENU:
+	case EEL_PREFERENCE_ITEM_ENUMERATION_MENU_INTEGER:
 		preferences_item_update_enumeration_menu (item);
 		break;
 
@@ -1314,8 +1325,8 @@ eel_preferences_item_set_description (EelPreferencesItem *item,
 	case EEL_PREFERENCE_ITEM_BOOLEAN:
 		g_assert (GTK_IS_CHECK_BUTTON (item->details->main_child));
 		g_assert (GTK_IS_LABEL (GTK_BIN (item->details->main_child)->child));
-		gtk_label_set_text (GTK_LABEL (GTK_BIN (item->details->main_child)->child),
-				    description);
+		gtk_label_set_text_with_mnemonic (GTK_LABEL (GTK_BIN (item->details->main_child)->child),
+						  description);
 		break;
 	case EEL_PREFERENCE_ITEM_ENUMERATION_VERTICAL_RADIO:
 	case EEL_PREFERENCE_ITEM_ENUMERATION_HORIZONTAL_RADIO:
@@ -1328,14 +1339,15 @@ eel_preferences_item_set_description (EelPreferencesItem *item,
 		g_assert (node != NULL);
 		g_assert (node->data != NULL);
 		g_assert (GTK_IS_LABEL (((GtkBoxChild *) node->data)->widget));
-		gtk_label_set_text (GTK_LABEL (((GtkBoxChild *) node->data)->widget),
-				    description);
+		gtk_label_set_text_with_mnemonic (GTK_LABEL (((GtkBoxChild *) node->data)->widget),
+						  description);
 		break;
 	case EEL_PREFERENCE_ITEM_FONT:
 	case EEL_PREFERENCE_ITEM_EDITABLE_PASSWORD_STRING:
 	case EEL_PREFERENCE_ITEM_EDITABLE_STRING:
 	case EEL_PREFERENCE_ITEM_EDITABLE_INTEGER:
 	case EEL_PREFERENCE_ITEM_ENUMERATION_MENU:
+	case EEL_PREFERENCE_ITEM_ENUMERATION_MENU_INTEGER:
 		g_assert (EEL_IS_CAPTION (item->details->main_child));
 		eel_caption_set_title_label (EEL_CAPTION (item->details->main_child),
 					     description);
